@@ -1,21 +1,29 @@
-import sqlalchemy as db
+from typing import ContextManager, Mapping, Optional
 
 import dagster._check as check
-from dagster.core.storage.config import pg_config
-from dagster.core.storage.runs import (
+import sqlalchemy as db
+import sqlalchemy.dialects as db_dialects
+import sqlalchemy.pool as db_pool
+from dagster._config.config_schema import UserConfigSchema
+from dagster._core.storage.config import PostgresStorageConfig, pg_config
+from dagster._core.storage.runs import (
     DaemonHeartbeatsTable,
     InstanceInfo,
     RunStorageSqlMetadata,
     SqlRunStorage,
 )
-from dagster.core.storage.sql import (
+from dagster._core.storage.runs.schema import KeyValueStoreTable
+from dagster._core.storage.sql import (
+    AlembicVersion,
     check_alembic_revision,
     create_engine,
     run_alembic_upgrade,
     stamp_alembic_rev,
 )
-from dagster.serdes import ConfigurableClass, ConfigurableClassData, serialize_dagster_namedtuple
-from dagster.utils import utc_datetime_from_timestamp
+from dagster._daemon.types import DaemonHeartbeat
+from dagster._serdes import ConfigurableClass, ConfigurableClassData, serialize_dagster_namedtuple
+from dagster._utils import utc_datetime_from_timestamp
+from sqlalchemy.engine import Connection
 
 from ..utils import (
     create_pg_connection,
@@ -34,8 +42,17 @@ class PostgresRunStorage(SqlRunStorage, ConfigurableClass):
     ``dagit`` and ``dagster-graphql`` load, based on the values in the ``dagster.yaml`` file in
     ``$DAGSTER_HOME``. Configuration of this class should be done by setting values in that file.
 
-    To use Postgres for run storage, you can add a block such as the following to your
-    ``dagster.yaml``:
+    To use Postgres for all of the components of your instance storage, you can add the following
+    block to your ``dagster.yaml``:
+
+    .. literalinclude:: ../../../../../../examples/docs_snippets/docs_snippets/deploying/dagster-pg.yaml
+       :caption: dagster.yaml
+       :lines: 1-8
+       :language: YAML
+
+    If you are configuring the different storage components separately and are specifically
+    configuring your run storage to use Postgres, you can add a block such as the following
+    to your ``dagster.yaml``:
 
     .. literalinclude:: ../../../../../../examples/docs_snippets/docs_snippets/deploying/dagster-pg-legacy.yaml
        :caption: dagster.yaml
@@ -46,7 +63,12 @@ class PostgresRunStorage(SqlRunStorage, ConfigurableClass):
     :py:class:`~dagster.IntSource` and can be configured from environment variables.
     """
 
-    def __init__(self, postgres_url, should_autocreate_tables=True, inst_data=None):
+    def __init__(
+        self,
+        postgres_url: str,
+        should_autocreate_tables: bool = True,
+        inst_data: Optional[ConfigurableClassData] = None,
+    ):
         self._inst_data = check.opt_inst_param(inst_data, "inst_data", ConfigurableClassData)
         self.postgres_url = postgres_url
         self.should_autocreate_tables = check.bool_param(
@@ -57,50 +79,59 @@ class PostgresRunStorage(SqlRunStorage, ConfigurableClass):
         self._engine = create_engine(
             self.postgres_url,
             isolation_level="AUTOCOMMIT",
-            poolclass=db.pool.NullPool,
+            poolclass=db_pool.NullPool,
         )
 
         self._index_migration_cache = {}
-        table_names = retry_pg_connection_fn(lambda: db.inspect(self._engine).get_table_names())
 
         # Stamp and create tables if the main table does not exist (we can't check alembic
         # revision because alembic config may be shared with other storage classes)
-        if self.should_autocreate_tables and "runs" not in table_names:
-            retry_pg_creation_fn(self._init_db)
-            self.migrate()
-            self.optimize()
-
-        elif "instance_info" not in table_names:
-            InstanceInfo.create(self._engine)
+        if self.should_autocreate_tables:
+            table_names = retry_pg_connection_fn(lambda: db.inspect(self._engine).get_table_names())
+            if "runs" not in table_names:
+                retry_pg_creation_fn(self._init_db)
+                self.migrate()
+                self.optimize()
+            elif "instance_info" not in table_names:
+                InstanceInfo.create(self._engine)
 
         super().__init__()
 
-    def _init_db(self):
+    def _init_db(self) -> None:
         with self.connect() as conn:
             with conn.begin():
                 RunStorageSqlMetadata.create_all(conn)
                 # This revision may be shared by any other dagster storage classes using the same DB
                 stamp_alembic_rev(pg_alembic_config(__file__), conn)
 
-    def optimize_for_dagit(self, statement_timeout):
+    def optimize_for_dagit(self, statement_timeout: int, pool_recycle: int) -> None:
         # When running in dagit, hold 1 open connection and set statement_timeout
+        existing_options = self._engine.url.query.get("options")
+        timeout_option = pg_statement_timeout(statement_timeout)
+        if existing_options:
+            options = f"{timeout_option} {existing_options}"
+        else:
+            options = timeout_option
         self._engine = create_engine(
             self.postgres_url,
             isolation_level="AUTOCOMMIT",
             pool_size=1,
-            connect_args={"options": pg_statement_timeout(statement_timeout)},
+            connect_args={"options": options},
+            pool_recycle=pool_recycle,
         )
 
     @property
-    def inst_data(self):
+    def inst_data(self) -> Optional[ConfigurableClassData]:
         return self._inst_data
 
     @classmethod
-    def config_type(cls):
+    def config_type(cls) -> UserConfigSchema:
         return pg_config()
 
     @staticmethod
-    def from_config_value(inst_data, config_value):
+    def from_config_value(
+        inst_data: Optional[ConfigurableClassData], config_value: PostgresStorageConfig
+    ):
         return PostgresRunStorage(
             inst_data=inst_data,
             postgres_url=pg_url_from_config(config_value),
@@ -108,9 +139,11 @@ class PostgresRunStorage(SqlRunStorage, ConfigurableClass):
         )
 
     @staticmethod
-    def create_clean_storage(postgres_url, should_autocreate_tables=True):
+    def create_clean_storage(
+        postgres_url: str, should_autocreate_tables: bool = True
+    ) -> "PostgresRunStorage":
         engine = create_engine(
-            postgres_url, isolation_level="AUTOCOMMIT", poolclass=db.pool.NullPool
+            postgres_url, isolation_level="AUTOCOMMIT", poolclass=db_pool.NullPool
         )
         try:
             RunStorageSqlMetadata.drop_all(engine)
@@ -118,31 +151,30 @@ class PostgresRunStorage(SqlRunStorage, ConfigurableClass):
             engine.dispose()
         return PostgresRunStorage(postgres_url, should_autocreate_tables)
 
-    def connect(self):
+    def connect(self) -> ContextManager[Connection]:
         return create_pg_connection(self._engine)
 
-    def upgrade(self):
+    def upgrade(self) -> None:
         with self.connect() as conn:
             run_alembic_upgrade(pg_alembic_config(__file__), conn)
 
-    def has_built_index(self, migration_name):
+    def has_built_index(self, migration_name: str) -> bool:
         if migration_name not in self._index_migration_cache:
             self._index_migration_cache[migration_name] = super(
                 PostgresRunStorage, self
             ).has_built_index(migration_name)
         return self._index_migration_cache[migration_name]
 
-    def mark_index_built(self, migration_name):
+    def mark_index_built(self, migration_name: str) -> None:
         super(PostgresRunStorage, self).mark_index_built(migration_name)
         if migration_name in self._index_migration_cache:
             del self._index_migration_cache[migration_name]
 
-    def add_daemon_heartbeat(self, daemon_heartbeat):
+    def add_daemon_heartbeat(self, daemon_heartbeat: DaemonHeartbeat) -> None:
         with self.connect() as conn:
-
             # insert or update if already present, using postgres specific on_conflict
             conn.execute(
-                db.dialects.postgresql.insert(DaemonHeartbeatsTable)
+                db_dialects.postgresql.insert(DaemonHeartbeatsTable)
                 .values(  # pylint: disable=no-value-for-parameter
                     timestamp=utc_datetime_from_timestamp(daemon_heartbeat.timestamp),
                     daemon_type=daemon_heartbeat.daemon_type,
@@ -159,7 +191,24 @@ class PostgresRunStorage(SqlRunStorage, ConfigurableClass):
                 )
             )
 
-    def alembic_version(self):
+    def kvs_set(self, pairs: Mapping[str, str]) -> None:
+        check.mapping_param(pairs, "pairs", key_type=str, value_type=str)
+
+        # pg speciic on_conflict_do_update
+        insert_stmt = db_dialects.postgresql.insert(KeyValueStoreTable).values(
+            [{"key": k, "value": v} for k, v in pairs.items()]
+        )
+        upsert_stmt = insert_stmt.on_conflict_do_update(
+            index_elements=[
+                KeyValueStoreTable.c.key,
+            ],
+            set_={"value": insert_stmt.excluded.value},
+        )
+
+        with self.connect() as conn:
+            conn.execute(upsert_stmt)
+
+    def alembic_version(self) -> AlembicVersion:
         alembic_config = pg_alembic_config(__file__)
         with self.connect() as conn:
             return check_alembic_revision(alembic_config, conn)
