@@ -1,17 +1,24 @@
-import pendulum
-import sqlalchemy as db
+from typing import ContextManager, Optional
 
 import dagster._check as check
-from dagster.core.storage.config import pg_config
-from dagster.core.storage.schedules import ScheduleStorageSqlMetadata, SqlScheduleStorage
-from dagster.core.storage.schedules.schema import InstigatorsTable
-from dagster.core.storage.sql import (
+import pendulum
+import sqlalchemy as db
+import sqlalchemy.dialects as db_dialects
+import sqlalchemy.pool as db_pool
+from dagster._config.config_schema import UserConfigSchema
+from dagster._core.scheduler.instigation import InstigatorState
+from dagster._core.storage.config import PostgresStorageConfig, pg_config
+from dagster._core.storage.schedules import ScheduleStorageSqlMetadata, SqlScheduleStorage
+from dagster._core.storage.schedules.schema import InstigatorsTable
+from dagster._core.storage.sql import (
+    AlembicVersion,
     check_alembic_revision,
     create_engine,
     run_alembic_upgrade,
     stamp_alembic_rev,
 )
-from dagster.serdes import ConfigurableClass, ConfigurableClassData, serialize_dagster_namedtuple
+from dagster._serdes import ConfigurableClass, ConfigurableClassData, serialize_dagster_namedtuple
+from sqlalchemy.engine import Connection
 
 from ..utils import (
     create_pg_connection,
@@ -30,8 +37,17 @@ class PostgresScheduleStorage(SqlScheduleStorage, ConfigurableClass):
     ``dagit`` and ``dagster-graphql`` load, based on the values in the ``dagster.yaml`` file in
     ``$DAGSTER_HOME``. Configuration of this class should be done by setting values in that file.
 
-    To use Postgres for schedule storage, you can add a block such as the following to your
-    ``dagster.yaml``:
+    To use Postgres for all of the components of your instance storage, you can add the following
+    block to your ``dagster.yaml``:
+
+    .. literalinclude:: ../../../../../../examples/docs_snippets/docs_snippets/deploying/dagster-pg.yaml
+       :caption: dagster.yaml
+       :lines: 1-8
+       :language: YAML
+
+    If you are configuring the different storage components separately and are specifically
+    configuring your schedule storage to use Postgres, you can add a block such as the following
+    to your ``dagster.yaml``:
 
     .. literalinclude:: ../../../../../../examples/docs_snippets/docs_snippets/deploying/dagster-pg-legacy.yaml
        :caption: dagster.yaml
@@ -42,7 +58,12 @@ class PostgresScheduleStorage(SqlScheduleStorage, ConfigurableClass):
     :py:class:`~dagster.IntSource` and can be configured from environment variables.
     """
 
-    def __init__(self, postgres_url, should_autocreate_tables=True, inst_data=None):
+    def __init__(
+        self,
+        postgres_url: str,
+        should_autocreate_tables: bool = True,
+        inst_data: Optional[ConfigurableClassData] = None,
+    ):
         self._inst_data = check.opt_inst_param(inst_data, "inst_data", ConfigurableClassData)
         self.postgres_url = postgres_url
         self.should_autocreate_tables = check.bool_param(
@@ -51,20 +72,20 @@ class PostgresScheduleStorage(SqlScheduleStorage, ConfigurableClass):
 
         # Default to not holding any connections open to prevent accumulating connections per DagsterInstance
         self._engine = create_engine(
-            self.postgres_url, isolation_level="AUTOCOMMIT", poolclass=db.pool.NullPool
+            self.postgres_url, isolation_level="AUTOCOMMIT", poolclass=db_pool.NullPool
         )
 
-        table_names = retry_pg_connection_fn(lambda: db.inspect(self._engine).get_table_names())
-
         # Stamp and create tables if the main table does not exist (we can't check alembic
         # revision because alembic config may be shared with other storage classes)
-        missing_main_table = "schedules" not in table_names and "jobs" not in table_names
-        if self.should_autocreate_tables and missing_main_table:
-            retry_pg_creation_fn(self._init_db)
+        if self.should_autocreate_tables:
+            table_names = retry_pg_connection_fn(lambda: db.inspect(self._engine).get_table_names())
+            missing_main_table = "schedules" not in table_names and "jobs" not in table_names
+            if missing_main_table:
+                retry_pg_creation_fn(self._init_db)
 
         super().__init__()
 
-    def _init_db(self):
+    def _init_db(self) -> None:
         with self.connect() as conn:
             with conn.begin():
                 ScheduleStorageSqlMetadata.create_all(conn)
@@ -74,25 +95,34 @@ class PostgresScheduleStorage(SqlScheduleStorage, ConfigurableClass):
         self.migrate()
         self.optimize()
 
-    def optimize_for_dagit(self, statement_timeout):
+    def optimize_for_dagit(self, statement_timeout: int, pool_recycle: int) -> None:
         # When running in dagit, hold an open connection and set statement_timeout
+        existing_options = self._engine.url.query.get("options")
+        timeout_option = pg_statement_timeout(statement_timeout)
+        if existing_options:
+            options = f"{timeout_option} {existing_options}"
+        else:
+            options = timeout_option
         self._engine = create_engine(
             self.postgres_url,
             isolation_level="AUTOCOMMIT",
             pool_size=1,
-            connect_args={"options": pg_statement_timeout(statement_timeout)},
+            connect_args={"options": options},
+            pool_recycle=pool_recycle,
         )
 
     @property
-    def inst_data(self):
+    def inst_data(self) -> Optional[ConfigurableClassData]:
         return self._inst_data
 
     @classmethod
-    def config_type(cls):
+    def config_type(cls) -> UserConfigSchema:
         return pg_config()
 
     @staticmethod
-    def from_config_value(inst_data, config_value):
+    def from_config_value(
+        inst_data: Optional[ConfigurableClassData], config_value: PostgresStorageConfig
+    ) -> "PostgresScheduleStorage":
         return PostgresScheduleStorage(
             inst_data=inst_data,
             postgres_url=pg_url_from_config(config_value),
@@ -100,9 +130,11 @@ class PostgresScheduleStorage(SqlScheduleStorage, ConfigurableClass):
         )
 
     @staticmethod
-    def create_clean_storage(postgres_url, should_autocreate_tables=True):
+    def create_clean_storage(
+        postgres_url: str, should_autocreate_tables: bool = True
+    ) -> "PostgresScheduleStorage":
         engine = create_engine(
-            postgres_url, isolation_level="AUTOCOMMIT", poolclass=db.pool.NullPool
+            postgres_url, isolation_level="AUTOCOMMIT", poolclass=db_pool.NullPool
         )
         try:
             ScheduleStorageSqlMetadata.drop_all(engine)
@@ -110,18 +142,18 @@ class PostgresScheduleStorage(SqlScheduleStorage, ConfigurableClass):
             engine.dispose()
         return PostgresScheduleStorage(postgres_url, should_autocreate_tables)
 
-    def connect(self, run_id=None):  # pylint: disable=arguments-differ, unused-argument
+    def connect(self, run_id: Optional[str] = None) -> ContextManager[Connection]:
         return create_pg_connection(self._engine)
 
-    def upgrade(self):
+    def upgrade(self) -> None:
         alembic_config = pg_alembic_config(__file__)
         with self.connect() as conn:
             run_alembic_upgrade(alembic_config, conn)
 
-    def _add_or_update_instigators_table(self, conn, state):
+    def _add_or_update_instigators_table(self, conn: Connection, state: InstigatorState) -> None:
         selector_id = state.selector_id
         conn.execute(
-            db.dialects.postgresql.insert(InstigatorsTable)
+            db_dialects.postgresql.insert(InstigatorsTable)
             .values(  # pylint: disable=no-value-for-parameter
                 selector_id=selector_id,
                 repository_selector_id=state.repository_selector_id,
@@ -140,7 +172,7 @@ class PostgresScheduleStorage(SqlScheduleStorage, ConfigurableClass):
             )
         )
 
-    def alembic_version(self):
+    def alembic_version(self) -> AlembicVersion:
         alembic_config = pg_alembic_config(__file__)
         with self.connect() as conn:
             return check_alembic_revision(alembic_config, conn)
